#!/usr/bin/perl -w

# LSB Test Execution Framework
# Configuration Page Module (tests_conf.pl)
#
# Copyright (C) 2007 The Linux Foundation. All rights reserved.
#
# This program has been developed by ISP RAS for LF.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

use Templates;

my @lsb_versions_list = qw/3.1 3.0/;

# %lsbv_arch:
# %(lsb_version => %(arch => %(module => %(PresV|AvailV|BetaV|PresF|AvailF|BetaF => version|version|version|@file_names|@file_names|@file_names))));
# %archs_list:
# %(arch => 1);
# %file_size:
# %(file_name => size);
# %mod_time:
# %(module_name => time);

my %lsbv_arch = ();
my %archs_list = ();
my %file_size = ();
my %mod_time = ();
my $ftp_list_expired = 0;

my %tests_options = ("cmdchk" => 3, "libchk" => 3, "desktop" => 3);

my $error_show = " display: none;";
my @error_msg = ();
my @warn_msg = ();

my %profile_data = (
	"name" => "",
	"organization" => "",
	"email" => "",
	"send_email" => "",
	"architecture" => "ia32",
	"use_internet" => " checked=\"checked\"",
	"keep_packages" => " checked=\"checked\"",
	"short_scen" => "",
	"lsb_version" => "3.1",
	"lsb_profile" => "desktop",
	"verbose_level" => "1",
	"comments" => ""
);

my $error_text = "";
my $warn_text = "";
my $empty_output = 1;

if (open(FILE, $CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list 2>/tmp/tests_error.$$ |")) {
	while (<FILE>) {
		$empty_output = 0;
		s/[\r\n]//g;
		if (/^ERROR: (.*)$/i) {
			$error_text .= $1."<br />";
		}
		elsif (/^WARNING: (.*)$/i) {
			$warn_text .= $1."<br />";
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^LSB_PROFILE:\s+(.*)$/) {
			$profile_data{"lsb_profile"} = $1;
		}
		elsif (/^EXPIRED:\s+(.*)$/) {
			$ftp_list_expired = 1;
		}
		elsif (/^FILE_SIZE:/) {
			my ($sig, $name, $sz) = split(/\s+/, $_, 3);
			$file_size{$name} = $sz;
		}
		elsif (/^MODULE_TIME:/) {
			my ($sig, $name, $tm) = split(/\s+/, $_, 3);
			$mod_time{$name} = $tm;
		}
		elsif (/^[^\s#]/) {
			my ($lsb_version, $arch, $mod, $stage, $vers, $files) = split(/\s+/, $_, 6);
			if ($stage eq "a") {
				$stage = "AVAIL";
			}
			elsif ($stage eq "b") {
				$stage = "BETA";
			}
			elsif ($stage eq "p") {
				$stage = "PRESENT";
			}
			my @file_names = (($files eq "") ? () : split(/,\s*/, $files));
			$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{$stage."_VERS"} = $vers;
			$lsbv_arch{$lsb_version}->{$arch}->{$mod}->{$stage."_FILES"} = \@file_names;
			$archs_list{$arch} = 1;
		}
	}
	close(FILE);
	if (-f "/tmp/tests_error.$$" and (-s "/tmp/tests_error.$$" != 0)) {
		if (!$empty_output) {
			$error_show = "";
			push @error_msg, "There were some errors while retrieving the list of available test modules!<br />See details in the <a href=\"get.pl$APP_DATA/log/server.log.8888\"><u>server log file</u></a>.";
		}
		print STDERR `cat /tmp/tests_error.$$`;
	}
	unlink("/tmp/tests_error.$$");
	if ($empty_output) {
		$error_show = "";
		push @error_msg, "Cannot get the list of available test modules: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
	}
	if ($error_text ne "") {
		$error_show = "";
		push @error_msg, $error_text."Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.<br />If you could not find any, please, write at <a href=\"mailto:fsgteam\@ispras.ru\"><u>fsgteam\@ispras.ru</u></a>.";
	}
	if ($warn_text ne "") {
		$error_show = "";
		push @warn_msg, $warn_text."Please, see the <a href=\"tests_help.pl#troubleshooting\"><u>Troubleshooting</u></a> Help section to find the solution.<br />If you could not find any, please, write at <a href=\"mailto:fsgteam\@ispras.ru\"><u>fsgteam\@ispras.ru</u></a>.";
	}
}
else {
	$error_show = "";
	push @error_msg, "Cannot get the list of available test modules: '".$CONFIG{"TESTS_DIR"}."/lsb-tef.pl -v0 --list'!<br />$!";
}

if (!-d "$APP_DATA/profiles") {
	system("mkdir -p $APP_DATA/profiles");
	if (!-d "$APP_DATA/profiles") {
		$error_show = "";
		push @error_msg, "Cannot not create the profiles directory!<br />$!";
	}
}

my $profile_name = "";
my $create_session_error = 0;

if (!$_COOKIE{"session_id"} or !-f "$APP_DATA/profiles/~session.$PeerIP.".$_COOKIE{"session_id"}) {
	my $i = time();
	while (-f "$APP_DATA/profiles/~session.$PeerIP.$i") {
		$i++;
	}
	$_COOKIE{"session_id"} = $i;
	$profile_name = "~session.$PeerIP.$i";
	if (open(FILE, "> $APP_DATA/profiles/$profile_name")) {
		close(FILE);
	}
	else {
		$error_show = "";
		push @error_msg, "Cannot create the session profile '$profile_name'<br />$!";
		$create_session_error = 1;
	}
}
else {
	$profile_name = "~session.$PeerIP.".$_COOKIE{"session_id"};
}

if (open(FILE, "$APP_DATA/profiles/$profile_name")) {
	my @comments = ();
	while (<FILE>) {
		s/[\r\n]//g;
		if (/^NAME:\s+(.*)$/) {
			$profile_data{"name"} = $1;
		}
		elsif (/^ORGANIZATION:\s+(.*)$/) {
			$profile_data{"organization"} = $1;
		}
		elsif (/^EMAIL:\s+(.*)$/) {
			$profile_data{"email"} = $1;
		}
		elsif (/^SEND_EMAIL:\s+(.*)$/) {
			$profile_data{"send_email"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^VERBOSE_LEVEL:\s+(.*)$/) {
			$profile_data{"verbose_level"} = $1;
		}
		elsif (/^KEEP_PACKAGES:\s+(.*)$/) {
			$profile_data{"keep_packages"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^SHORT_SCENARIOS:\s+(.*)$/) {
			$profile_data{"short_scen"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^LSB_VERSION:\s+(.*)$/) {
			$profile_data{"lsb_version"} = $1;
		}
		elsif (/^LSB_PROFILE:\s+(.*)$/) {
			$profile_data{"lsb_profile"} = $1;
		}
		elsif (/^ARCHITECTURE:\s+(.*)$/) {
			$profile_data{"architecture"} = $1;
		}
		elsif (/^USE_INTERNET:\s+(.*)$/) {
			$profile_data{"use_internet"} = (($1 eq "1") ? " checked=\"checked\"" : "");
		}
		elsif (/^COMMENTS: (.*)$/) {
			push @comments, $1;
		}
		elsif (/^[^\s#]/) {
			my ($mod, $stage) = split(/\s+/, $_, 2);
			if ($mod =~ m/^(.*)-opt$/) {
				$tests_options{$1} = $stage;
			}
			else {
				$profile_data{"modules"}->{$mod} = $stage;
			}
		}
	}
	close(FILE);
	$profile_data{"comments"} = join("\n", @comments);
	if ($profile_data{"email"} eq "") {
		$profile_data{"send_email"} = "";
	}
}
elsif (!$create_session_error) {
	$error_show = "";
	push @error_msg, "Cannot read the session profile data: $profile_name!<br />$!";
}

if ($_COOKIE{"last_error_text"}) {
	$error_show = "";
	push @error_msg, $_COOKIE{"last_error_text"};
}

print "HTTP/1.0 200 OK\n";
print "Content-type: text/html\n";
print "Set-Cookie: session_id=".$_COOKIE{"session_id"}."; path=/;\n";
print "Set-Cookie: last_error_text=; path=/;\n\n";

print_header("Configure Custom Tests", "conf");

if ((scalar(@error_msg) != 0) or (scalar(@warn_msg) != 0)) {
	$error_text = "<ul>";
	foreach (@error_msg) {
		$error_text .= "<li><font color=\"red\"><u><b>Error:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	foreach (@warn_msg) {
		$error_text .= "<li><font color=\"darkred\"><u><b>Warning:</b></u></font> <font color=\"darkblue\">$_</font></li>";
	}
	$error_text .= "</ul>";
}

$table_spacer = "<td><img src=\"images/environment-spacer.gif\" width=\"5\" height=\"1\" border=\"0\" alt=\"\" /></td>";

print <<DATA;
<noscript><h1 align="center"><font color="red" size="+1">WARNING! Your browser does not support JavaScript.</font></h1></noscript>
<div id="error_msg_collapsed" style="border: dashed 1px darkred; background-color: red; float: right; cursor: pointer; display: none;" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='none';" title="Expand">
<table border="0" cellpadding="1" cellspacing="1">
  <tr>
    <td align="left" style="white-space: nowrap;"><font color="white"><b>&nbsp;There are some problems!</b></font></td>
    <td width="15px"></td>
    <td width="11px"><img src="images/expand.png" alt="O" width="20" height="20" /></td>
  </tr>
</table>
</div>
<div id="error_msg_area" style="border: none;$error_show">
<table border="0" cellpadding="1" cellspacing="1" width="70%" align="center">
  <tr>
    <th style="border: dashed 1px darkred; background: red; color: white; font-size: large;">
      <table border="0" cellpadding="0" cellspacing="0" align="center">
        <tr>
          <td width="100%">Errors!</td>
          <td><img src="images/collapse.png" alt="_" width="20" height="20" style="cursor: pointer;" title="Collapse" onclick="javascript:var e=document.getElementById('error_msg_area');if(e) e.style.display='none';e=document.getElementById('error_msg_collapsed');if(e) e.style.display='';" /></td>
        </tr>
      </table>
    </th>
  </tr>
  <tr><td align="center" style="border: dashed 1px darkred; border-top: none;">
    <table border="0" cellpadding="0" cellspacing="0">
      <tr><td height="10px" colspan="3"></td></tr>
      <tr><td width="50%"></td><td align="left" style="font-size: medium; white-space: nowrap;">$error_text</td><td width="50%"></td></tr>
      <tr><td height="10px" colspan="3"></td></tr>
    </table>
  </td></tr>
</table>
</div>

<div id="tooltip" style="display: none; background-color: #ffffd0; border: solid 1px black; position: absolute;">
<table cellpadding="0" cellspacing="0" border="0" width="200">
  <tr style="background-color: #d0d0d0; color: black;">
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/environment-spacer.gif" width="7" height="1" alt="" /></td>
    <td align="center" width="100%" style="border-bottom: solid 1px black;"><b>Information</b></td>
    <td align="right" style="border-bottom: solid 1px black;"><img src="images/close.png" onclick="javascript:hide_tooltip();" width="20" height="20" title="Close" alt="X" style="cursor: pointer;" /></td>
  </tr>
  <tr>
    <td colspan="3" style="padding: 5px; white-space: nowrap;" id="tooltip_text"></td>
  </tr>
</table>
</div>

<h1>Configure Custom Tests</h1>
<form method="post" action="tests_conf_save.pl" name="configure_test">
<input type="hidden" name="session_id" value="$_COOKIE{session_id}" />
<input type="hidden" name="form_action" id="form_action" value="" />
<input type="hidden" name="lsb_profile" id="lsb_profile" value="$profile_data{lsb_profile}" />
<table border="0" cellpadding="0" cellspacing="0">
  <tr>
    <td valign="top" width="50%">
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td>Your&nbsp;name:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="tester_name" value="$profile_data{name}" style="width: 150px;" /></td>
        </tr>
        <tr>
          <td>Organization:</td>
          $table_spacer
          <td colspan="2"><input type="text" name="organization" value="$profile_data{organization}" style="width: 150px;" /></td>
        </tr>
        <tr>
          <td>E-mail:</td>
          $table_spacer
          <td><input type="text" name="tester_email" value="$profile_data{email}" style="width: 150px;" /></td>
          <td width="100%" style="white-space: nowrap;"><label><input type="checkbox" name="send_email_report" id="send_email_report"$profile_data{send_email} /> Send report via E-mail</label></td>
        </tr>
        <tr>
          <td colspan="4" style="border: dashed 1px gray; background: #ffc000; cursor: pointer;" onclick="javascript:hide_tooltip();show_hide_details();">&nbsp;<img src="images/arr_r.png" alt="" id="block_arrow_img" width="12" height="12" />&nbsp;&nbsp;Additional options</td>
        </tr>
        <tr>
          <td colspan="4" id="additional_data" style="border: dashed 1px gray; border-top: none; display: none;">
            <table border="0" cellpadding="2" cellspacing="2" width="100%">
              <tr>
                <td colspan="2"><label><input type="checkbox" name="keep_packages" id="keep_packages"$profile_data{keep_packages} /> Keep packages</label>&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#keep_packages" onclick="javascript:show_hide_tooltip('keep_packages', event); return false;">?</a>]</b></td>
              </tr>
              <tr>
                <td colspan="2"><label><input type="checkbox" name="short_scen" id="short_scen"$profile_data{short_scen} /> Short scenarios</label>&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#short_scenarios" onclick="javascript:show_hide_tooltip('short_scenarios', event); return false;">?</a>]</b></td>
              </tr>
              <tr>
                <td width="127px">Verbose Level:</td>
                <td width="393px">
                  <select name="verbose_level" style="width: 156px;">
DATA
print "                    <option value=\"1\"".(($profile_data{verbose_level} == 1) ? " selected=\"selected\"" : "").">1 (normal)</option>\n";
print "                    <option value=\"2\"".(($profile_data{verbose_level} == 2) ? " selected=\"selected\"" : "").">2 (high)</option>\n";
print <<DATA;
                  </select>
                </td>
              </tr>
              <tr>
                <td colspan="2">Comments:</td>
              </tr>
              <tr>
                <td colspan="2">
                  <textarea name="comments" rows="5" cols="64">$profile_data{comments}</textarea>
                </td>
              </tr>
            </table>
          </td>
        </tr>
        <tr>
          <td colspan="4" valign="bottom" height="15"><hr /></td>
        </tr>
        <tr>
          <td colspan="4" valign="top" style="white-space: nowrap;"><label><input type="checkbox" name="use_internet" id="use_internet" checked="checked" onclick="javascript:update_elements_state();" ondblclick="javascript:setTimeout('update_elements_state()', 500);" />&nbsp;&nbsp;Use Internet for downloading the tests (download size: <b><span id="download_sz">0&nbsp;Kbytes</span></b>)</label></td>
        </tr>
        <tr>
          <td valign="middle" colspan="4">
            <table border="0" cellpadding="0" cellspacing="0" width="100%">
              <tr>
                <td>Architecture:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td>
                  <select name="architecture" id="arch_select" style="width: 80px;" onchange="javascript:show_modules();">
DATA

if (scalar(keys(%archs_list)) == 0) {
	print "                    <option value=\"$profile_data{architecture}\" selected=\"selected\">$profile_data{architecture}</option>\n";
}
else {
	foreach (sort keys %archs_list) {
		if ($_ eq $profile_data{"architecture"}) {
			print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
		}
		else {
			print "                    <option value=\"$_\">$_</option>\n";
		}
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="10" height="1" border="0" alt="" /></td>
                <td>LSB&nbsp;Version:</td>
                <td><img src="images/environment-spacer.gif" width="5" height="1" border="0" alt="" /></td>
                <td>
                  <select name="lsb_version" id="lsbv_select" style="width: 80px;" onchange="javascript:show_modules();">
DATA

foreach (@lsb_versions_list) {
	if ($_ eq $profile_data{"lsb_version"}) {
		print "                    <option value=\"$_\" selected=\"selected\">$_</option>\n";
	}
	else {
		print "                    <option value=\"$_\">$_</option>\n";
	}
}

print <<DATA;
                  </select>
                </td>
                <td><img src="images/environment-spacer.gif" width="100" height="1" border="0" alt="" /></td>
DATA

my $exp_style = "";
my $exp_warning = "";
my $refresh_button_name = ((scalar(keys(%archs_list)) == 0) ? "Get List" : "Refresh List");

if (scalar(keys(%archs_list)) == 0) {
	$exp_style = " color: red; font-weight: bold;";
	$exp_warning = "            <tr>\n              <td colspan=\"4\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">The list was not obtained yet! Press \"$refresh_button_name\" button.</font></b></td>\n            </tr>\n";
}
elsif ($ftp_list_expired) {
	$exp_style = " color: red; font-weight: bold;";
	$exp_warning = "            <tr>\n              <td colspan=\"4\" align=\"center\"><b><font color=\"red\">WARNING:</font> <font color=\"blue\">The list may be outdated! Press \"$refresh_button_name\" button.</font></b></td>\n            </tr>\n";
}

if (scalar(keys(%archs_list)) == 0) {
	print "                <td align=\"right\" width=\"100%\"><input type=\"submit\" name=\"refresh_list\" id=\"refresh_list_button\" value=\"Get List\" style=\"width: 100px; color: red; font-weight: bold;\" title=\"Get the list of available tests from the Internet\" onclick=\"javascript:return do_action('refresh_list');\" /></td>\n";
}
else {
	print "                <td align=\"right\" width=\"100%\"><input type=\"submit\" name=\"refresh_list\" id=\"refresh_list_button\" value=\"Refresh List\" style=\"width: 100px;$exp_style\" title=\"Refresh the list of available tests from the Internet\" onclick=\"javascript:return do_action('refresh_list');\" /></td>\n";
}

print <<DATA;
              </tr>
            </table>
          </td>
        </tr>
$exp_warning      </table>
DATA

my $lsb_v;

sub print_test_suite_row($$$$) {
	my ($bkgd, $arch, $mod, $lsb_v) = @_;
	my $x1 = $lsbv_arch{$lsb_v};
	my $x2 = $x1->{$arch};
	my $x3 = $x2->{$mod};
	my $stage = $lsbv_arch{$lsb_v}->{$arch}->{$mod};
	my $mod_pp = $mod;
	$mod_pp =~ s/\+/p/g;
	print "        <tr class=\"list_item$bkgd\">\n          <td><label><input type=\"checkbox\" name=\"mod-$arch-$lsb_v-$mod_pp\" id=\"mod-$arch-$lsb_v-$mod_pp\" onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$mod</label></td>\n";
	my $statep = " disabled=\"disabled\"".($stage->{"PRESENT_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $statea = " disabled=\"disabled\"".($stage->{"AVAIL_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $stateb = " disabled=\"disabled\"".($stage->{"BETA_VERS"} ? " onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\"" : "");
	my $idp = $stage->{"PRESENT_VERS"} ? "-p" : "-p-";
	my $ida = $stage->{"AVAIL_VERS"} ? "-a" : "-a-";
	my $idb = $stage->{"BETA_VERS"} ? "-b" : "-b-";
	if ($stage->{"PRESENT_VERS"}) {
		$statep = " checked=\"checked\"".$statep;
	}
	elsif ($stage->{"AVAIL_VERS"}) {
		$statea = " checked=\"checked\"".$statea;
	}
	elsif ($stage->{"BETA_VERS"}) {
		$stateb = " checked=\"checked\"".$stateb;
	}
	print "          <td><label><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$idp\" value=\"p\"$statep /> ".($stage->{"PRESENT_VERS"} or "")."</label></td>\n";
	print "          <td><label><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$ida\" value=\"a\"$statea /> ".($stage->{"AVAIL_VERS"} or "")."</label></td>\n";
	print "          <td><label><input type=\"radio\" name=\"modstage-$arch-$lsb_v-$mod_pp\" id=\"modstage-$arch-$lsb_v-$mod_pp$idb\" value=\"b\"$stateb /> ".($stage->{"BETA_VERS"} or "")."</label></td>\n        </tr>\n";
	if ($mod eq "desktop") {
		my $checked3 = (($tests_options{$mod} == 3) ? " checked=\"checked\"" : "");
		my $checked4 = (($tests_options{$mod} == 4) ? " checked=\"checked\"" : "");
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"3\" id=\"modopt-$arch-$lsb_v-$mod_pp-3\" disabled=\"disabled\"$checked3 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Desktop only</label></td>\n        </tr>\n";
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"4\" id=\"modopt-$arch-$lsb_v-$mod_pp-4\" disabled=\"disabled\"$checked4 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Desktop &amp; Qt4 (<i>optional</i>)</label></td>\n        </tr>\n";
	}
	elsif ($mod eq "libchk") {
		my $checked2 = (($tests_options{$mod} == 2) ? " checked=\"checked\"" : "");
		my $checked3 = (($tests_options{$mod} == 3) ? " checked=\"checked\"" : "");
		my $checked4 = (($tests_options{$mod} == 4) ? " checked=\"checked\"" : "");
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"2\" id=\"modopt-$arch-$lsb_v-$mod_pp-2\" disabled=\"disabled\"$checked2 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Core &amp; C++</label></td>\n        </tr>\n";
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"3\" id=\"modopt-$arch-$lsb_v-$mod_pp-3\" disabled=\"disabled\"$checked3 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Core &amp; C++ &amp; Desktop</label></td>\n        </tr>\n";
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"4\" id=\"modopt-$arch-$lsb_v-$mod_pp-4\" disabled=\"disabled\"$checked4 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Core &amp; C++ &amp; Desktop &amp; Qt4 (<i>optional</i>)</label></td>\n        </tr>\n";
	}
	elsif ($mod eq "cmdchk") {
		my $checked2 = (($tests_options{$mod} == 2) ? " checked=\"checked\"" : "");
		my $checked3 = (($tests_options{$mod} == 3) ? " checked=\"checked\"" : "");
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"2\" id=\"modopt-$arch-$lsb_v-$mod_pp-2\" disabled=\"disabled\"$checked2 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Core &amp; C++</label></td>\n        </tr>\n";
		print "        <tr class=\"list_item$bkgd\">\n          <td colspan=\"4\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<label><input type=\"radio\" name=\"modopt-$arch-$lsb_v-$mod_pp\" value=\"3\" id=\"modopt-$arch-$lsb_v-$mod_pp-3\" disabled=\"disabled\"$checked3 onclick=\"javascript:update_elements_state();\" ondblclick=\"javascript:setTimeout('update_elements_state()', 500);\" />&nbsp;Core &amp; C++ &amp; Desktop</label></td>\n        </tr>\n";
	}
}

foreach $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %{$lsbv_arch{$lsb_v}}) {
		if (($arch eq $profile_data{"architecture"}) and ($lsb_v eq $profile_data{"lsb_version"})) {
			print "      <table id=\"arch-$arch-$lsb_v\" cellpadding=\"0\" cellspacing=\"0\" style=\"border: solid 4px white; width: 100%;\">\n";
		}
		else {
			print "      <table id=\"arch-$arch-$lsb_v\" cellpadding=\"0\" cellspacing=\"0\" style=\"border: solid 4px white; width: 100%; display: none;\">\n";
		}
		my $count;
		my %listed = ();
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			$listed{$mod} = 1;
		}
		print <<DATA;
        <tr class="list_head">
          <td width="170px"><input type="checkbox" name="check-all-$arch-$lsb_v" id="check-all-$arch-$lsb_v" onclick="javascript:un_check_all('all');" /></td>
          <td width="120px"><b>Localhost</b></td>
          <td width="120px"><b>FTP</b></td>
          <td width="120px"><b>FTP (beta)</b></td>
        </tr>
        <tr class="list_cat">
          <td colspan="4"><b><label><input type="checkbox" name="check-cert-$arch-$lsb_v" id="check-cert-$arch-$lsb_v" onclick="javascript:un_check_all('cert');" />&nbsp;Certification Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#cert" onclick="javascript:show_hide_tooltip('cert', event); return false;">?</a>]</b></td>
        </tr>
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-cmdtests-$arch-$lsb_v" id="check-cmdtests-$arch-$lsb_v" onclick="javascript:un_check_all('cmdtests');" />&nbsp;&nbsp;&nbsp;Command Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#cmdtests" onclick="javascript:show_hide_tooltip('cmdtests', event); return false;">?</a>]</b></td>
        </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/cmdchk/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-sitests-$arch-$lsb_v" id="check-sitests-$arch-$lsb_v" onclick="javascript:un_check_all('sitests');" />&nbsp;&nbsp;&nbsp;Static Interface Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#sitests" onclick="javascript:show_hide_tooltip('sitests', event); return false;">?</a>]</b></td>
        </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/libchk/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-ritests-$arch-$lsb_v" id="check-ritests-$arch-$lsb_v" onclick="javascript:un_check_all('ritests');" />&nbsp;&nbsp;&nbsp;Runtime Interface Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#ritests" onclick="javascript:show_hide_tooltip('ritests', event); return false;">?</a>]</b></td>
        </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/desktop libstdc++ runtime vsw4/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-abtests-$arch-$lsb_v" id="check-abtests-$arch-$lsb_v" onclick="javascript:un_check_all('abtests');" />&nbsp;&nbsp;&nbsp;Automated Application Battery Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#abtests" onclick="javascript:show_hide_tooltip('abtests', event); return false;">?</a>]</b></td>
        </tr>
DATA
		$count = 0;
	  	foreach $mod (qw/apache expect groff lynx python raptor rsync samba tcl xpdf/) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}

		print <<DATA;
        <tr class="list_cat">
          <td colspan="4"><b><label><input type="checkbox" name="check-unoff-$arch-$lsb_v" id="check-unoff-$arch-$lsb_v" onclick="javascript:un_check_all('unoff');" />&nbsp;Unofficial Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#unoff" onclick="javascript:show_hide_tooltip('unoff', event); return false;">?</a>]</b></td>
        </tr>
        <tr class="list_subcat">
          <td colspan="4"><b><label><input type="checkbox" name="check-uritests-$arch-$lsb_v" id="check-uritests-$arch-$lsb_v" onclick="javascript:un_check_all('uritests');" />&nbsp;&nbsp;&nbsp;Runtime Interface Tests</label>&nbsp;&nbsp;&nbsp;[<a href="tests_help.pl#uritests" onclick="javascript:show_hide_tooltip('uritests', event); return false;">?</a>]</b></td>
        </tr>
DATA
		$count = 0;
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			if ($listed{$mod}) {
				$count++;
				print_test_suite_row(($count % 2), $arch, $mod, $lsb_v);
				delete $listed{$mod};
			}
		}
		print "      </table>\n";
	}
}

print "        <br />\n";

if (scalar(keys(%archs_list)) != 0) {
	print <<DATA;
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        &nbsp;<b><a href="" onclick="javascript:var e=document.getElementById('warning');if (e) {e.style.display=(e.style.display=='none'?'':'none');} return false;" title="Click here for details"><font color="red">WARNING!</font> <font color="green">Do not use the tests on any machine holding important data!</font></a></b><br />
        <div id="warning" style="display: none;"><br />
        <div style="border: 1px dashed #2f6fab; background-color: #f9f9f9; width: 90%; padding: 3px; font-family: monospace;">
          WARNING! PLEASE DO NOT USE THIS SCRIPT ON ANY MACHINE THAT HOLDS IMPORTANT DATA! The tests sometimes put a heavy stress on the tested computer, and in rare unfortunate cases they can crash the system or damage the stored data. They also are a security risk as they pass along the root password.
        </div>
        </div>
      </td></tr></table>
DATA
}

print <<DATA;
      <table cellpadding="0" cellspacing="0" width="100%" style="border: solid 4px white;"><tr><td>
        <input type="submit" name="run_tests" id="run_tests_button" value="Run selected tests" style="width: 150px;" onclick="javascript:return do_action('run_tests');" /><!--&nbsp;&nbsp; (Estimated&nbsp;execution&nbsp;time:&nbsp;<b><span id="estim_time">0&nbsp;min</span></b>)-->
      </td></tr></table>
    </td>
    <td style="border-right: dashed 1px gray;"><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
    <td><img src="images/environment-spacer.gif" width="15" height="1" border="0" alt="" /></td>
DATA

my $profiles_options = "";
my @profiles_list;
if (opendir(DIR, "$APP_DATA/profiles")) {
	@profiles_list = sort grep !/^[~\.]/, readdir(DIR);
	foreach (@profiles_list) {
		$profiles_options .= "              <option value=\"$_\">$_</option>\n";
	}
	closedir(DIR);
}

my $disabled_list = "";
if ($profiles_options eq "") {
	$profiles_options = "              <option value=\"\">&lt;No profiles present&gt;</option>\n";
	$disabled_list = " disabled=\"disabled\"";
}

my $profiles_list_num = scalar(@profiles_list);
if ($profiles_list_num < 2) {
	$profiles_list_num = 2;
}
elsif ($profiles_list_num > 6) {
	$profiles_list_num = 6;
}

my $current_lsb = $profile_data{"lsb_version"};
my $current_arch = $profile_data{"architecture"};

print <<DATA;
    <td valign="top" width="50%">
      <h2>User Profile Management&nbsp;&nbsp;&nbsp;<b>[<a href="tests_help.pl#profile_management" onclick="javascript:show_hide_tooltip('profile_management', event); return false;">?</a>]</b></h2>
      <table border="0" cellpadding="2" cellspacing="2" width="100%">
        <tr>
          <td colspan="3">Select the profile or type its name:</td>
        </tr>
        <tr>
          <td colspan="3">
            <table border="0" cellpadding="0" cellspacing="0"><tr>
              <td style="vertical-align: middle;">
                <input type="text" name="edit_profile_name" id="edit_profile_name" value="" style="width: 200px;" onkeypress="javascript:return onKeyPressEdit(event);" />
              </td>
              <td style="vertical-align: middle;">
                <img src="images/drop_down.png" alt="v" width="16" height="20" id="drop_down_button" style="cursor: pointer;" onclick="javascript:onClickArrow();" />
              </td>
            </tr></table>
            <div id="hidden_div" style="position: absolute; z-index: +1; display: none;">
              <select name="list_profile_name" id="list_profile_name" size="$profiles_list_num" style="width: 222px;"$disabled_list onclick="javascript:onClickList();" onkeypress="javascript:return onKeyPressList(event)">
$profiles_options              </select>
            </div>
          </td>
        </tr>
        <tr>
          <td><input type="submit" name="save_profile_button" id="save_profile_button" value="Save" style="width: 70px;" onclick="javascript:return do_action('save_profile');" /></td>
          <td><input type="submit" name="load_profile_button" id="load_profile_button" value="Load" style="width: 70px;" onclick="javascript:return do_action('load_profile');" /></td>
          <td width="100%"><input type="submit" name="delete_profile_button" id="delete_profile_button" value="Delete" style="width: 70px;" onclick="javascript:return do_action('delete_profile');" /></td>
        </tr>
      </table>
    </td>
  </tr>
</table>
</form>
<script language="javascript" type="text/javascript">
// <![CDATA[
var preload_image_names = new Array('arr_d.png', 'expand.png', 'collapse.png', 'drop_down_pressed.png', 'close.png');
var preload_images = new Array();
for (var i=0; i<preload_image_names.length; ++i) {
	preload_images[i] = new Image;
	preload_images[i].src = 'images/' + preload_image_names[i];
}
var arch_mod = new Array();
var file_size = new Array();
var mod_time = new Array();
var current_arch = "$current_arch-$current_lsb";
DATA

foreach my $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %archs_list) {
		my $mods = $lsbv_arch{$lsb_v}->{$arch};
		print "arch_mod[\"$arch-$lsb_v\"] = new Array();\n";
		foreach my $mod (sort keys %$mods) {
			my $stage = $mods->{$mod};
			my $mod_pp = $mod;
			$mod_pp =~ s/\+/p/g;
			print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"] = new Array();\n";
			if ($stage->{"PRESENT_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"PRESENT_VERS\"] = \"".$stage->{"PRESENT_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"PRESENT_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"PRESENT_FILES"}})."\");\n";
			}
			if ($stage->{"AVAIL_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"AVAIL_VERS\"] = \"".$stage->{"AVAIL_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"AVAIL_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"AVAIL_FILES"}})."\");\n";
			}
			if ($stage->{"BETA_VERS"}) {
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"BETA_VERS\"] = \"".$stage->{"BETA_VERS"}."\";\n";
				print "arch_mod[\"$arch-$lsb_v\"][\"$mod_pp\"][\"BETA_FILES\"] = new Array(\"".join("\", \"", @{$stage->{"BETA_FILES"}})."\");\n";
			}
		}
	}
}

foreach my $fname (sort keys %file_size) {
	print "file_size[\"$fname\"] = \"".$file_size{$fname}."\";\n";
}
print "file_size[\"\"] = \"0\";\n";

foreach my $mname (sort keys %mod_time) {
	print "mod_time[\"$mname\"] = \"".$mod_time{$mname}."\";\n";
}

print "var arch_list = new Array(\"".join("\", \"", keys %archs_list)."\");\n";
print "var lsbv_list = new Array(\"".join("\", \"", @lsb_versions_list)."\");\n";

my %mod_list = ();
foreach $lsb_v (@lsb_versions_list) {
	foreach my $arch (sort keys %{$lsbv_arch{$lsb_v}}) {
		foreach my $mod (sort keys %{$lsbv_arch{$lsb_v}->{$arch}}) {
			$mod =~ s/\+/p/g;
			$mod_list{$mod} = 1;
		}
	}
}

print "var mod_list = new Array(\"".join("\", \"", sort keys %mod_list)."\");\n\n";

print <<DATA;
var edit_profile_name = document.getElementById('edit_profile_name');
var list_profile_name = document.getElementById('list_profile_name');
var hidden_div = document.getElementById('hidden_div');
var drop_down_button = document.getElementById('drop_down_button');
var valid_drop_box = list_profile_name && edit_profile_name && hidden_div && drop_down_button;

function mergeArrays(arr1, arr2) {
	var res = arr1;
	var i, j, k = res.length;
	var found;
	for (i=0; i<arr2.length; ++i) {
		found = 0;
		for (j=0; j<arr1.length; ++j) {
			if (arr1[j] == arr2[i]) {
				found = 1;
				break;
			}
		}
		if (!found)
			res[k++] = arr2[i];
	}
	return res;
}

function update_elements_state() {
	var elem;
	var checked;
	var i, j;
	var files = new Array();
	var total_sz = 0;
	var total_time = 0;
	var total_num = 0;

	// Enable/disable "Refresh List" button (as it needs Internet connection)
	elem = document.getElementById("use_internet");
	var checked_internet = elem ? elem.checked : 0;
	elem = document.getElementById("refresh_list_button");
	if (elem)
		elem.disabled = !checked_internet;

	// for each module...
	for (i=0; i<mod_list.length; ++i) {
		var elemp, elema, elemb, elemopt;
		var enablep, enablea, enableb;
		// Find the module's checkbox ...
		elem = document.getElementById("mod-" + current_arch + "-" + mod_list[i]);
		if (!elem)
			continue;
		checked = elem.checked;
		// ... radio-buttons of the options ...
		for (j=2; j<=4; j++) {
			elemopt = document.getElementById("modopt-" + current_arch + "-" + mod_list[i] + "-" + j);
			if (elemopt)
				elemopt.disabled = !checked;
		}
		// ... and radio-button of stages (present/available/beta).
		elemp = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-p");
		elema = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-a");
		elemb = document.getElementById("modstage-" + current_arch + "-" + mod_list[i] + "-b");
		if (checked_internet) {
			// If Internet is possible, then no problems.
			enablep = true;
			enablea = true;
			enableb = true;
		}
		else {
			// Else we find out if the Internet connection is needed for downloading anything
			// for each of the stage-variants (enable[p|a|b]==1 if there is nothing to download).
			var tmp_files;
			if (elemp) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["PRESENT_FILES"];
				enablep = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enablep = false;
						break;
					}
				}
			}
			else
				enablep = false;
			if (elema) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["AVAIL_FILES"];
				enablea = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enablea = false;
						break;
					}
				}
			}
			else
				enablea = false;
			if (elemb) {
				tmp_files = arch_mod[current_arch][mod_list[i]]["BETA_FILES"];
				enableb = true;
				for (j=0; j<tmp_files.length; ++j) {
					if (file_size[tmp_files[j]] != 0) {
						enableb = false;
						break;
					}
				}
			}
			else
				enableb = false;
		}
		if (enablep || enablea || enableb) {
			// If at least one variant is permitted, allow the module, ...
			elem.disabled = false;
			// ... and the stage-variants, if needed.
			enablep = enablep && checked;
			enablea = enablea && checked;
			enableb = enableb && checked;
			if (elemp)
				elemp.disabled = !enablep;
			if (elema)
				elema.disabled = !enablea;
			if (elemb)
				elemb.disabled = !enableb;
			if (checked) {
				// Change stage to possible, if the selected one is prohibited.
				var preferrable;
				if (elemp && !elemp.disabled) {
					preferrable = elemp;
				}
				else if (elema && !elema.disabled) {
					preferrable = elema;
				}
				else if (elemb && !elemb.disabled) {
					preferrable = elemb;
				}
				if (((elemp && elemp.checked && elemp.disabled) ||
					 (elema && elema.checked && elema.disabled) ||
					 (elemb && elemb.checked && elemb.disabled)) && preferrable)
					preferrable.checked = true;
				// Now add the files needed for downloading to the global list.
				if (elemp && elemp.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["PRESENT_FILES"]);
				}
				if (elema && elema.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["AVAIL_FILES"]);
				}
				if (elemb && elemb.checked) {
					files = mergeArrays(files, arch_mod[current_arch][mod_list[i]]["BETA_FILES"]);
				}
				++total_num;
				//total_time += parseInt(mod_time[mod_list[i]]);
			}
		}
		else {
			// If no variant is possible, turn the whole module off.
			elem.disabled = true;
			if (elemp)
				elemp.disabled = true;
			if (elema)
				elema.disabled = true;
			if (elemb)
				elemb.disabled = true;
		}
	}
	// Summarize the size of files to download...
	for (i=0; i<files.length; ++i) {
		total_sz += parseInt(file_size[files[i]]);
	}
	// and display this size together with the total time of running the tests.
	document.getElementById("download_sz").innerHTML = Math.round(total_sz / 1024) + "&nbsp;Kbytes";
	//document.getElementById("estim_time").innerHTML =  Math.round(total_time / 60) + "&nbsp;min ";
	elem = document.getElementById("run_tests_button");
	if (elem)
		elem.disabled = (total_num == 0);
}

var mod_cat_list = new Array();
mod_cat_list["all"] = new Array("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf", "xts5");
mod_cat_list["cert"] = new Array("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
mod_cat_list["cmdtests"] = new Array("cmdchk");
mod_cat_list["sitests"] = new Array("libchk");
mod_cat_list["ritests"] = new Array("desktop", "libstdcpp", "runtime", "vsw4");
mod_cat_list["abtests"] = new Array("apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
mod_cat_list["unoff"] = new Array("xts5");
mod_cat_list["uritests"] = new Array("xts5");

var check_list = new Array();
check_list["all"] = new Array("cert", "cmdtests", "sitests", "ritests", "abtests", "unoff", "uritests");
check_list["cert"] = new Array("cmdtests", "sitests", "ritests", "abtests");
check_list["cmdtests"] = new Array();
check_list["sitests"] = new Array();
check_list["ritests"] = new Array();
check_list["abtests"] = new Array();
check_list["unoff"] = new Array("uritests");
check_list["uritests"] = new Array();

function un_check_all(category) {
	var elem = document.getElementById("check-" + category + "-" + current_arch);
	if (elem) {
		var checked = elem.checked;
		for (var i=0; i<mod_cat_list[category].length; ++i) {
			elem = document.getElementById("mod-" + current_arch + "-" + mod_cat_list[category][i]);
			if (elem)
				elem.checked = checked;
		}
		for (var i=0; i<check_list[category].length; ++i) {
			elem = document.getElementById("check-" + check_list[category][i] + "-" + current_arch);
			if (elem)
				elem.checked = checked;
		}
		if ((category == "cert") && checked) {
			elem = document.getElementById("modopt-" + current_arch + "-libchk-3");
			if (elem)
				elem.checked = true;
			elem = document.getElementById("modopt-" + current_arch + "-desktop-3");
			if (elem)
				elem.checked = true;
			elem = document.getElementById("modopt-" + current_arch + "-cmdchk-3");
			if (elem)
				elem.checked = true;
		}
		update_elements_state();
	}
}

function show_modules() {
	var i, j;
	var elem_a = document.getElementById("arch_select");
	var elem_l = document.getElementById("lsbv_select");
	if (!elem_a || !elem_l)
		return;
	var arch_lsb = elem_a.value + "-" + elem_l.value;

	for (i=0; i<lsbv_list.length; i++) {
		for (j=0; j<arch_list.length; j++) {
			var tmp = arch_list[j] + "-" + lsbv_list[i];
			var elem = document.getElementById("arch-" + tmp);
			if (elem) {
				if (tmp == arch_lsb) {
					elem.style.display = "";
					current_arch = tmp;
				}
				else {
					elem.style.display = "none";
				}
			}
		}
	}
	update_elements_state();
}

function show_hide_details() {
	var elem_div = document.getElementById("additional_data");
	var elem_img = document.getElementById("block_arrow_img");
	if (elem_div && elem_img) {
		var to_show = (elem_div.style.display == "none");
		elem_div.style.display = to_show ? "" : "none";
		elem_img.setAttribute("src", to_show ? "images/arr_d.png" : "images/arr_r.png");
	}
}

DATA

my $array_profiles = ((scalar(@profiles_list) > 0) ? "'".join("', '", @profiles_list)."'" : "");

print <<DATA;
function do_action(name) {
	update_elements_state();
	var elem = document.getElementById("form_action");
	if (!elem) {
		alert("Internal error: Cannot find 'form_action' field!\\nPlease, contact the authors.");
		return false;
	}
	if (name == "delete_profile") {
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (edit_profile_name.value) {
			if (!confirm("Do you really want to delete the profile '" + edit_profile_name.value + "'?"))
				return false;
		}
		else {
			alert("Please, specify the profile name!");
			return false;
		}
	}
	else if (name == "load_profile") {
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (!edit_profile_name.value) {
			alert("Please, specify the profile name!");
			return false;
		}
	}
	else if (name == "save_profile") {
		var profile_names = new Array($array_profiles);
		if (!edit_profile_name){
			alert("Internal error: Cannot find 'edit_profile_name' field!\\nPlease, contact the authors.");
			return false;
		}
		if (edit_profile_name.value == "") {
			alert("Please, specify the profile name!");
			return false;
		}
		else {
			for (var i=0; i<profile_names.length; ++i) {
				if (edit_profile_name.value == profile_names[i]) {
					if (!confirm("Do you want to overwrite the profile '" + edit_profile_name.value + "'?"))
						return false;
					break;
				}
			}
		}
	}
	elem.value = name;
	return true;
}

function check_profile_modules() {
    var to_check = new Array();
    var to_check_cat = new Array();
DATA

my %mod_cat_list = ();
@{$mod_cat_list{"all"}} = ("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf", "xts5");
@{$mod_cat_list{"cert"}} = ("cmdchk", "libchk", "desktop", "libstdcpp", "runtime", "vsw4", "apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
@{$mod_cat_list{"cmdtests"}} = ("cmdchk");
@{$mod_cat_list{"sitests"}} = ("libchk");
@{$mod_cat_list{"ritests"}} = ("desktop", "libstdcpp", "runtime", "vsw4");
@{$mod_cat_list{"abtests"}} = ("apache", "expect", "groff", "lynx", "python", "raptor", "rsync", "samba", "tcl", "xpdf");
@{$mod_cat_list{"unoff"}} = ("xts5");
@{$mod_cat_list{"uritests"}} = ("xts5");

foreach my $mod (keys %{$profile_data{"modules"}}) {
	my $stage = $profile_data{"modules"}->{$mod};
	print "\tto_check[\"$current_arch-$current_lsb-$mod\"] = \"$stage\";\n"
}

foreach my $cat (keys %mod_cat_list) {
	my $full = 1;
	foreach (@{$mod_cat_list{$cat}}) {
		if (!($profile_data{"modules"}->{$_})) {
			$full = 0;
			last;
		}
	}
	if ($full) {
		print "\tto_check_cat[\"$cat\"] = 1;\n"
	}
}

print <<DATA;
	for (var mod in to_check) {
		var elem = document.getElementById("mod-" + mod);
		if (elem) {
			elem.checked = 1;
		}
		elem = document.getElementById("modstage-" + mod + "-" + to_check[mod]);
		if (elem) {
			elem.checked = 1;
		}
    }
	for (var cat in to_check_cat) {
		var elem = document.getElementById("check-" + cat + "-$current_arch-$current_lsb");
		if (elem) {
			elem.checked = 1;
		}
	}
}

var last_tooltip = '';
var tooltip_texts = new Array();
tooltip_texts['keep_packages'] = 'If this option is turned on, the test packages<br /> remain installed in the system after the test execution.';
tooltip_texts['short_scenarios'] = 'This option is for debug purpose only!<br />It allows to run a reduced set of tests,<br />which will result in much shorter time of executing.';
tooltip_texts['cert'] = 'Test suites that are used for certification of distributions<br />for conformance to the LSB Specifications.';
tooltip_texts['cmdtests'] = 'Test suites that verify the commands<br />required by LSB specification.';
tooltip_texts['sitests'] = 'Test suites that verify interfaces in the runtime libraries.';
tooltip_texts['ritests'] = 'Test suites for testing LSB runtime environments<br />for conformance to the LSB Specifications.';
tooltip_texts['abtests'] = 'Tests that verify the correct operation of binary applications<br />from the LSB application battery.';
tooltip_texts['unoff'] = 'Tests that are not used for certification,<br />but can be used for futher LSB compliance testing.';
tooltip_texts['uritests'] = 'Test suites for testing LSB runtime environments<br />for conformance to the LSB Specifications.';
tooltip_texts['profile_management'] = 'This section allows you to save all the settings<br />from the left pane into user profiles and load them afterwards.<br />See <a href="tests_help.pl#profile_management" target="_blank" onclick="javascript:hide_tooltip();"><b>Help</b></a> for details.';

var just_shown_tooltip = false;

function show_hide_tooltip(id, event) {
	var tt_window = document.getElementById("tooltip");
	if (tt_window) {
		if (last_tooltip != id) {
			var tt_text = document.getElementById("tooltip_text");
			if (tt_text) {
				tt_text.innerHTML = (tooltip_texts[id] || "<font color=\\"red\\"><b>Sorry, no information for this item was found!</b></font><br />Please, write to <a href=\\"mailto:fsgteam\@ispras.ru\\">fsgteam\@ispras.ru</a>.");
				var ScrollTop = document.body.scrollTop;
				if (ScrollTop == 0)
				{
				    if (window.pageYOffset)
				        ScrollTop = window.pageYOffset;
				    else
				        ScrollTop = (document.body.parentElement) ? document.body.parentElement.scrollTop : 0;
				}
				var ScrollLeft = document.body.scrollLeft;
				if (ScrollLeft == 0)
				{
				    if (window.pageXOffset)
				        ScrollLeft = window.pageXOffset;
				    else
				        ScrollLeft = (document.body.parentElement) ? document.body.parentElement.scrollLeft : 0;
				}
				var div_top = event.clientY + ScrollTop + 10;
				var div_left = event.clientX + ScrollLeft + 10;
				tt_window.style.display = '';
				var div_width = (tt_window.style.pixelWidth || tt_window.offsetWidth);
				var wnd_width = (document.body.clientWidth || window.innerWidth);
				if (div_left + div_width > wnd_width)
					div_left = wnd_width - div_width;
				tt_window.style.top = div_top + 'px';
				tt_window.style.left = div_left + 'px';
				last_tooltip = id;
				just_shown_tooltip = true;
			}
		}
		else {
			tt_window.style.display = 'none';
			last_tooltip = '';
		}
	}
}

function hide_tooltip() {
	var elem = document.getElementById("tooltip");
	if (elem) {
		elem.style.display = 'none';
		last_tooltip = '';
	}
}

var VK_ESC  = 27;
var VK_CR   = 13;
var VK_LF   = 10;
var VK_DOWN = 40;
var VK_TAB  = 9;

var just_shown_list = false;

function show_list() {
	hidden_div.style.display = '';
	drop_down_button.setAttribute("src", "images/drop_down_pressed.png");
	just_shown_list = true;
}

function hide_list(set_focus) {
	if (set_focus)
		edit_profile_name.focus();
	hidden_div.style.display = 'none';
	drop_down_button.setAttribute("src", "images/drop_down.png");
}

function onBodyClick() {
	if (valid_drop_box && (hidden_div.style.display != 'none')) {
		if (!just_shown_list)
			hide_list(false);
		just_shown_list = false;
	}
	if (!just_shown_tooltip)
		hide_tooltip();
	just_shown_tooltip = false;
}

function onKeyPressList(event) {
	if (!valid_drop_box || (hidden_div.style.display == 'none'))
		return true;
	if (event.keyCode == VK_ESC) {
		hide_list(true);
		return false;
	}
	else if ((event.keyCode == VK_CR) || (event.keyCode == VK_LF)) {
		edit_profile_name.value = list_profile_name.value;
		hide_list(true);
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		hide_list(false)
		return true;
	}
	return true;
}

function onClickList() {
	if (!valid_drop_box || (hidden_div.style.display == 'none'))
		return true;
	edit_profile_name.value = list_profile_name.value;
	hide_list(true);
}

function onKeyPressEdit(event) {
	if (!valid_drop_box)
		return true;
	if (event.keyCode == VK_ESC) {
		if (hidden_div.style.display != 'none') {
			hide_list(true);
			return false;
		}
	}
	else if (event.keyCode == VK_DOWN) {
		if (hidden_div.style.display == 'none') {
			show_list();
		}
		list_profile_name.focus();
		return false;
	}
	else if (event.keyCode == VK_TAB) {
		if (hidden_div.style.display != 'none') {
			list_profile_name.focus();
			return false;
		}
	}
	else
		hide_list(true);
	return true;
}

function onClickArrow() {
	if (!valid_drop_box)
		return true;
	if (hidden_div.style.display == 'none') {
		show_list();
		list_profile_name.focus();
	}
	else {
		hide_list(true);
	}
}

function save_and_go(url) {
	if (window.event.shiftKey)
		document.configure_test.target = "_blank";
	document.configure_test.action = "tests_conf_save.pl?goto=" + url.replace("#", "%23");
	var elem = document.getElementById("form_action");
	if (elem) {
		elem.value = name;
		document.configure_test.submit();
		return false;
	}
	else
		return true;
}

function modify_all_links() {
	var links = document.getElementsByTagName("A");
	for (var i = 0; i < links.length; i++) {
		if ((links[i].target == "") && (links[i].href != "") && (links[i].onclick == null)) {
			links[i].onclick = new Function("return save_and_go(\\"" + links[i].href + "\\");");
		}
	}
}

modify_all_links();
check_profile_modules();
update_elements_state();
// ]]>
</script>
DATA

print_footer();
